/* TODO
  allow for multiple locations
  allow user to use their photos
  fix midnight time issue
  add translations
  add notifications
  if location error, pull to refresh issue
  hide keyboard when clicking go button on menu
  update time on time format change
  display error if not all field are valid on menu close
  show hourly on tablet
  context animated icons
*/

angular.module('starter.controllers', [])

    /**** Factories ****/
    .factory('LocationFactory', function () {

        var getLocation = function (callback) {

            var platform = device.platform.toLowerCase();

                // is location authorized
                if (platform === "android") {
                    // check permissions
                    cordova.plugins.diagnostic.isLocationAuthorized(function (authorized) {
                        if (authorized) {
                            console.log("permissions authorized");
                            doGetLocation();
                        } else {
                            console.log("request permissions");
                            // request permissions
                            cordova.plugins.diagnostic.requestLocationAuthorization(function(status){
                                switch(status){
                                    case cordova.plugins.diagnostic.permissionStatus.NOT_REQUESTED:
                                        console.log("Permission not requested");
                                        break;
                                    case cordova.plugins.diagnostic.permissionStatus.GRANTED:
                                        console.log("Permission granted");
                                        doGetLocation();
                                        break;
                                    case cordova.plugins.diagnostic.permissionStatus.DENIED:
                                        console.log("Permission denied");
                                        doGetLocation();
                                        break;
                                    case cordova.plugins.diagnostic.permissionStatus.DENIED_ALWAYS:
                                        console.log("Permission permanently denied");
                                        doGetLocation();
                                        break;
                                }
                            }, function(error){
                                console.error(error);
                            });
                        }
                    }, function(error){
                        console.error("The following error occurred: "+error);
                    });
                } else {
                    doGetLocation();
                }

            function doGetLocation() {
                var onSuccess = function(position) {
                    clearTimeout(timeout);
                    callback(position.coords.latitude + "," + position.coords.longitude);
                };

                // onError Callback receives a PositionError object
                var onError = function (error) {
                    alert('code: ' + error.code + '\n' +
                        'message: ' + error.message + '\n');
                    clearTimeout(timeout);
                    callback(null);
                };

                if (navigator.geolocation) {
                    var timeout = setTimeout(callback, 12000),
                        posOptions = {
                            timeout: 12000,
                            enableHighAccuracy: false
                        };
                    navigator.geolocation.getCurrentPosition(onSuccess, onError);
                } else {
                    callback(null);
                }
            }
        };
        return {
            getLocation: getLocation
        };
    })

    .factory('AddressFactory', function($http) {
        var API_URL = "https://maps.googleapis.com/maps/api/geocode/json";
        var getAddress = function(location, callback) {

            $http({
                method: 'GET',
                url: API_URL,
                params: {
                    address: location,
                    key: 'AIzaSyBzLeJP_tVDZFOesDSSQLGgIDbeouVqXXY'
                }
            })
                .then(function successCallback(response) {
                    if (response.data.results.length > 0) {
                        var results = response.data.results[0],
                            location = results.formatted_address,
                            locationArray = location.split(',');
                        while (locationArray.length > 3) {
                            locationArray.shift();
                        }
                        var address = locationArray.join(', ');
                        var latLon = results.geometry.location.lat + "," + results.geometry.location.lng;
                        callback(address, latLon);
                    } else {
                        callback(null);
                    }
                }, function errorCallback(response) {
                    callback(null);
                });
        };
        return {
            getAddress: getAddress
        };
    })

    .factory('SettingsFactory', function($window) {
        return {
            set: function(key, value) {
                $window.localStorage[key] = value;
            },
            get: function(key, defaultValue) {
                return $window.localStorage[key] || defaultValue;
            },
            setObject: function(key, value) {
                $window.localStorage[key] = JSON.stringify(value);
            },
            getObject: function(key) {
                return JSON.parse($window.localStorage[key] || '{}');
            }
        };
    })

    .factory('WeatherFactory', function($http) {
        var FORECAST_URL = "https://api.forecast.io/forecast/3bbd290adef3a40741f516fae3123729/";

        function getURL(latLon) {
            var url = FORECAST_URL + latLon;

            console.log(url);
            return url;
        }

        var getWeatherData = function(settings, latLon, callback) {
            $http({
                method: 'GET',
                url: getURL(latLon),
                params: {
                    "cache": false,
                    "timeout": "9000",
                    "units": settings.scale,
                    "exclude": "minutely",
                    "extend": "hourly"
                }
            })
                .then(function successCallback(response) {
                    callback(response.data);
                }, function errorCallback(response) {
                    callback(null);
                });
        };
        return {
            getWeatherData: getWeatherData
        };
    }) // End Factories

    /**** Controller ****/
    .controller('AppCtrl', function(
        $scope, $filter, $ionicLoading, $ionicSideMenuDelegate, $ionicModal, $ionicPlatform, $ionicScrollDelegate,
        WeatherFactory, SettingsFactory, AddressFactory, LocationFactory) {

        // With the new view caching in Ionic, Controllers are only called
        // when they are recreated or on app start, instead of every page change.
        // To listen for when this page is active (for example, to refresh data),
        // listen for the $ionicView.enter event:
        //$scope.$on('$ionicView.enter', function(e) {
        //});

        $scope.settings = {};
        $scope.locationError = false;
        $scope.weatherData = {};
        $scope.temperatureUnit = '';
        $scope.windSpeedUnitShort = '';
        $scope.windSpeedUnitLong = '';
        $scope.latLon = '';
        $scope.address = '';
        $scope.hourly = null;
        $scope.dailySelected = null;

        var init = true,
            location = "";

        /**** Settings ****/

        function getSettings() {
            console.log('getSettings');
            $scope.settings = SettingsFactory.getObject('weatherSettings');
            var setSettingsCalled = false;

            // If settings not found in localstorage
            if (angular.equals({}, $scope.settings)) {
                $scope.settings = {
                    scale: "auto",
                    temperatureLow: 40,
                    temperatureHigh: 95,
                    precipitationLow: 0,
                    precipitationHigh: 50,
                    windLow: 0,
                    windHigh: 15,
                    timeFormat: 12,
                    timeStart: 6,
                    timeEnd: 20,
                    location: ""
                };
                setSettingsCalled = true;
                setSettings();
            } else {
                $scope.settings = SettingsFactory.getObject('weatherSettings');
            }
            if ($scope.settings.location === "" && !setSettingsCalled) {
                getLocation();
            } else {
                location = $scope.settings.location;
                getAddress();
            }
        }

        function setSettings() {
            console.log('setSettings');
            SettingsFactory.setObject('weatherSettings', $scope.settings);
            if ($scope.settings.location === "") {
                getLocation();
            } else {
                location = $scope.settings.location;
                getAddress();
            }
        }

        /**** Address ****/
        function setAddress(address, latLon) {
            console.log('setAddress');
            if (address) {
                $scope.address = address;
                $scope.latLon = latLon;
                getWeather();
            } else {
                $scope.address = '';
                loadingHide();
                $scope.locationError = true;
            }
        }

        function getAddress() {
            console.log('getAddress');
            var location;
            if ($scope.settings.location !== "") {
                location = $scope.settings.location;
            } else {
                if ($scope.latLon !== "") {
                    location = $scope.latLon;
                } else {
                    getLocation();
                    return;
                }
            }
            AddressFactory.getAddress(location, setAddress);
        }

        /**** Location ****/
        function setLocation(latLon) {
            console.log('setLocation');
            if (latLon) {
                $scope.latLon = latLon;
                getAddress();
            } else {
                loadingHide();
                $scope.locationError = true;
            }
        }

        function getLocation(callback) {
            console.log('getLocation');
            LocationFactory.getLocation(setLocation);
        }

        function setUnits() {
            console.log('setUnits');
            if ($scope.weatherData.flags.units === 'us') {
                $scope.temperatureUnit = 'F';
                $scope.windSpeedUnit = 'mph';
            } else if ($scope.weatherData.flags.units === 'si' ||
                $scope.weatherData.flags.units === 'ca' ||
                $scope.weatherData.flags.units === 'uk2') {
                $scope.temperatureUnit = 'C';
                $scope.windSpeedUnit = 'mps';
            }
            if ($scope.weatherData.flags.units === 'ca') {
                $scope.windSpeedUnit = 'kph';
            }
            if ($scope.weatherData.flags.units === 'uk2') {
                $scope.windSpeedUnit = 'mph';
            }
        }

        function scrollTop() {
            $ionicScrollDelegate.$getByHandle('mainScroll').scrollTop(true);
        }

        /**** Share ****/
        var onShareSuccess = function(result) {
            console.log("Share completed? " + result.completed); // On Android apps mostly return false even while it's true
            console.log("Shared to app: " + result.app); // On Android result.app is currently empty. On iOS it's empty when sharing is cancelled (result.completed=false)
        };

        var onShareError = function(msg) {
            console.log("Sharing failed with message: " + msg);
        };

        $scope.share = function() {
            // this is the complete list of currently supported params you can pass to the plugin (all optional)
            var options = {
                message: 'Get the Jeep Weather App for iOS or Android. A weather app designed for fishing enthusists.', // not supported on some apps (Facebook, Instagram)
                subject: 'Jeep Weather', // fi. for email
                // files: ['', ''], // an array of filenames either locally or remotely
                url: 'https://www.Jeep-Weather.com/'
                // chooserTitle: 'Pick an app' // Android only, you can override the default share sheet title
            };
            window.plugins.socialsharing.shareWithOptions(options, onShareSuccess, onShareError);
        };

        /**** Weather ****/
        function setWeather(weatherData) {
            console.log('setWeather');
            loadingHide();
            $scope.$broadcast('scroll.refreshComplete');
            if (weatherData) {
                weatherData = setDaily(weatherData);
                $scope.weatherData = weatherData;
                setUnits();
            } else {
                $scope.connectionError = true;
            }
        }

        function getWeather() {
            console.log('getWeather');
            $scope.weatherData = {};
            $scope.locationError = false;
            $scope.connectionError = false;
            WeatherFactory.getWeatherData($scope.settings, $scope.latLon, setWeather);
            scrollTop();
        }

        /**** UI Helpers ****/
        $scope.getPrecipProbabilityIcon = function(precipProbability, precipType) {
            if (precipProbability >= 0.01 && precipType) {
                return 'wi wi-' + precipType;
            } else {
                return 'icon ion-minus';
            }
        };

        function between(x, minNum, maxNum) {
            return x >= minNum && x <= maxNum;
        }

        $scope.getWeatherIcon = function(weatherData) {
            if (weatherData && weatherData.temperature) {
                weatherData.temperatureMin = weatherData.temperature;
                weatherData.temperatureMax = weatherData.temperature;
            }
            if (weatherData && weatherData.temperatureMin >= parseInt($scope.settings.temperatureLow, 10) &&
                weatherData.temperatureMax <= parseInt($scope.settings.temperatureHigh, 10) &&
                weatherData.precipProbability * 100 >= parseInt($scope.settings.precipitationLow, 10) &&
                weatherData.precipProbability * 100 <= parseInt($scope.settings.precipitationHigh, 10) &&
                weatherData.windSpeed >= parseInt($scope.settings.windLow, 10) &&
                weatherData.windSpeed <= parseInt($scope.settings.windHigh, 10)) {
                return "good";
            } else {
                return "bad";
            }
        };

        $scope.getWindBearing = function(windBearing) {
            if (windBearing) {
                if (between(windBearing, 337.6, 360) || between(windBearing, 0, 22.5)) return "south";
                if (between(windBearing, 22.6, 67.5)) return "south-west";
                if (between(windBearing, 67.6, 112.5)) return "west";
                if (between(windBearing, 112.6, 157.5)) return "north-west";
                if (between(windBearing, 157.6, 205.5)) return "north";
                if (between(windBearing, 205.6, 247.5)) return "north-east";
                if (between(windBearing, 247.6, 292.5)) return "east";
                if (between(windBearing, 292.6, 337.5)) return "south-east";
            } else {
                return "";
            }
        };

        $scope.hourlyBetween = function(min, max) {
            return function(hourly) {
                var hourlyTime = parseInt($filter('date')(hourly.time * 1000, "H"));
                return between(hourlyTime, min, max);
            };
        };

        $scope.isHourly = function(hourly) {
            if ($scope.hourly === hourly) {
                return true;
            } else {
                return false;
            }
        };

        $scope.hideToday = function(daily) {
            var nowHours = parseInt($filter('date')(new Date(), "HH"));
            if (new Date().getDate() === new Date(daily.time * 1000).getDate() &&
                nowHours > $scope.settings.timeEnd) {
                return true;
            } else {
                return false;
            }
        };

        /**** Weather Alert Modal ****/
        $scope.modalData = {};

        $ionicModal.fromTemplateUrl('alert-modal.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function(modal) {
            $scope.modal = modal;
        });
        $scope.openModal = function(alert) {
            $scope.modalData = alert;
            $scope.modal.show();
        };
        $scope.closeModal = function() {
            $scope.modal.hide();
        };

        function setDaily(weatherData) {
            console.log('setDaily');
            var dailyArray = {},
                hourlyArray = [];

            angular.forEach(weatherData.hourly.data, function(item, i) {
                var time = parseInt($filter('date')(item.time * 1000, "HH"));
                if (between(time, $scope.settings.timeStart, $scope.settings.timeEnd)) {
                    var id = $filter('date')(item.time * 1000, 'yyyyMMdd');

                    if (!dailyArray[id]) {
                        dailyArray[id] = {};
                    }

                    // Daily Weather Info
                    dailyArray[id].time = item.time;
                    if (!dailyArray[id].temperatureMax || dailyArray[id].temperatureMax < item.temperature) {
                        dailyArray[id].temperatureMax = item.temperature;
                    }
                    if (!dailyArray[id].temperatureMin || dailyArray[id].temperatureMin > item.temperature) {
                        dailyArray[id].temperatureMin = item.temperature;
                    }
                    if (!dailyArray[id].precipProbability || dailyArray[id].precipProbability < item.precipProbability) {
                        dailyArray[id].precipProbability = item.precipProbability;
                    }
                    if (!dailyArray[id].windSpeed || dailyArray[id].windSpeed < item.windSpeed) {
                        dailyArray[id].windSpeed = item.windSpeed;
                    }

                    if (!dailyArray[id].precipType && item.precipType) {
                        dailyArray[id].precipType = item.precipType;
                    } else if (item.precipType === 'hail') {
                        dailyArray[id].precipType = item.precipType;
                    } else if (dailyArray[id].precipType !== 'hail' && item.precipType === 'sleet') {
                        dailyArray[id].precipType = item.precipType;
                    } else if (dailyArray[id].precipType !== 'hail' && dailyArray[id].precipType !== 'sleet' && item.precipType === 'snow') {
                        dailyArray[id].precipType = item.precipType;
                    } else if (dailyArray[id].precipType !== 'hail' && dailyArray[id].precipType !== 'sleet' && dailyArray[id].precipType !== 'snow' && item.precipType === 'rain') {
                        dailyArray[id].precipType = item.precipType;
                    }
                    hourlyArray.push(item);
                }
            });

            var count = 0,
                tempArray = [];

            angular.forEach(dailyArray, function(item, i) {
                var dailyObj = {};
                dailyObj.time = dailyArray[i].time;
                dailyObj.summary = weatherData.daily.data[count].summary;
                dailyObj.sunriseTime = weatherData.daily.data[count].sunriseTime;
                dailyObj.sunsetTime = weatherData.daily.data[count].sunsetTime;
                dailyObj.sunsetTime = weatherData.daily.data[count].sunsetTime;
                dailyObj.windBearing = weatherData.daily.data[count].windBearing;
                dailyObj.temperatureMax = dailyArray[i].temperatureMax;
                dailyObj.temperatureMin = dailyArray[i].temperatureMin;
                dailyObj.precipProbability = dailyArray[i].precipProbability;
                dailyObj.windSpeed = dailyArray[i].windSpeed;
                if (dailyArray[i].precipType) {
                    dailyObj.precipType = dailyArray[i].precipType;
                } else {
                    dailyObj.precipType = null;
                }
                tempArray.push(dailyObj);
                count++;
            });
            weatherData.daily.data = tempArray;
            weatherData.hourly.data = hourlyArray;

            return weatherData;
        }

        $scope.showHourly = function(daily) {
            $scope.dailySelected = daily;
            if (daily === null) {
                $scope.hourly = null;
            } else {
                $scope.hourly = parseInt($filter('date')(daily.time * 1000, "yyyyMMdd"));
            }
            scrollTop();
        };

        /**** Event Handlers ****/
        $scope.$watch('settings.scale', function(newValue, oldValue) {
            if (newValue !== oldValue) {
                loadingShow();
                setSettings();
            }
        });

        /**** Menu Close Event Handler ****/
        $scope.$watch(function() {
            return $ionicSideMenuDelegate.isOpenRight();
        },
            function(isOpen) {
                if (isOpen || init) {
                    init = false;
                } else {
                    $scope.weatherData = {};
                    $scope.locationError = false;
                    $scope.connectionError = false;
                    loadingShow();
                    setSettings();
                }
            });

        $scope.openMenu = function() {
            $ionicSideMenuDelegate.toggleRight();
        };

        $ionicPlatform.registerBackButtonAction(function(e) {
            e.preventDefault();
            if ($ionicSideMenuDelegate.isOpenRight()) {
                $ionicSideMenuDelegate.toggleRight();
            } else if ($scope.hourly !== null) {
                $scope.showHourly(null);
                $scope.$apply();
            } else {
                navigator.app.exitApp();
            }
        }, 1000);

        $scope.doRefresh = function() {
            getWeather();
        };

        var loadingShow = function() {
            $ionicLoading.show({
                template: '<ion-spinner></ion-spinner><br>Loading...'
            });
        };

        var loadingHide = function() {
            $ionicLoading.hide();
        };

        loadingShow();
        getSettings();

    }); // End controller
